/* 
   FBColumnMetaData.m

   Copyright (C) 1999 MDlink online service center GmbH and Helge Hess

   Author: Helge Hess (helge.hess@mdlink.de)

   This file is part of the FB Adaptor Library

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; see the file COPYING.LIB.
   If not, write to the Free Software Foundation,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
// $Id: FBColumnMetaData.m,v 1.2 1999/11/02 23:32:40 helge Exp $

#include "FBColumnMetaData.h"
#include <FBCAccess/FBCAccess.h>
#import <Foundation/Foundation.h>

@implementation FBColumnMetaData

- (id)initWithComponents:(NSString *)_str {
  if ((self->cmd = fbccmdMetaDataWithComponents((char *)[_str cString]))==NULL) {
    NSLog(@"Couldn't create column meta data from '%@'", _str);
    RELEASE(self);
    return nil;
  }
  return self;
}

- (id)initWithCColumnMetaData:(void *)_ccmd {
  if (_ccmd == NULL) {
    NSLog(@"Missing column meta data !");
    RELEASE(self);
    return nil;
  }
  self->cmd = _ccmd;
  return self;
}

- (void)dealloc {
  if (self->cmd) {
    fbccmdRelease(self->cmd);
    self->cmd = NULL;
  }
  [super dealloc];
}

/* accessors */

- (NSString *)schemaName {
  char *s;

  return ((s = fbccmdSchemaName(self->cmd)))
    ? [NSString stringWithCString:s]
    : nil;
}

- (NSString *)tableName {
  char *s;

  return ((s = fbccmdTableName(self->cmd)))
    ? [NSString stringWithCString:s]
    : nil;
}

- (NSString *)columnName {
  char *s;

  return ((s = fbccmdColumnName(self->cmd)))
    ? [NSString stringWithCString:s]
    : nil;
}

- (NSString *)labelName {
  char *s;

  return ((s = fbccmdLabelName(self->cmd)))
    ? [NSString stringWithCString:s]
    : nil;
}

- (BOOL)isNullable {
  return fbccmdIsNullable(self->cmd) ? YES : NO;
}

/* description */

- (NSString *)description {
  return [NSString stringWithFormat:
                     @"<%@[0x%08X]:"
                     @" schema=%@ table=%@ column=%@ label=%@"
                     @" isNullable=%s",
                     NSStringFromClass([self class]), self,
                     [self schemaName],
                     [self tableName],
                     [self columnName],
                     [self labelName],
                     [self isNullable] ? "yes" : "no"];
}

@end /* FBColumnMetaData */
