/* 
   FBDatabaseConnection.m

   Copyright (C) 1999 MDlink online service center GmbH and Helge Hess

   Author: Helge Hess (helge.hess@mdlink.de)

   This file is part of the FB Adaptor Library

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; see the file COPYING.LIB.
   If not, write to the Free Software Foundation,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
// $Id: FBDatabaseConnection.m,v 1.2 1999/11/02 23:32:40 helge Exp $

#include "FBDatabaseConnection.h"
#include "FBMetaData.h"
#include "FBBlobHandle.h"
#include "FrontBaseException.h"
#include <FBCAccess/FBCAccess.h>
#import <Foundation/Foundation.h>
#include <unistd.h>

@implementation FBDatabaseConnection

- (BOOL)setRollbackAfterError:(BOOL)_flag {
  return fbcdcSetRollbackAfterError(self->fb, _flag ? Yes : No) ? YES : NO;
}

- (void)dealloc {
  if (self->fb) [self close];
  [super dealloc];
}

- (void)connectToDatabaseNamed:(NSString *)_db {
  [self connectToDatabaseNamed:_db onHost:nil password:nil];
}
- (void)connectToDatabaseNamed:(NSString *)_db onHost:(NSString *)_hostName {
  [self connectToDatabaseNamed:_db onHost:_hostName password:nil];
}

- (void)connectToDatabaseNamed:(NSString *)_db
  onHost:(NSString *)_hostName
  password:(NSString *)_dbPwd
{
  /* disconnect if open */
  if (self->fb) [self close];
  
  if (_hostName == nil)
    _hostName = [[NSHost currentHost] name];

  self->fb = fbcdcConnectToDatabase((char *)[_db       cString],
                                    (char *)[_hostName cString],
                                    (char *)[_dbPwd    cString]);
  if (self->fb == NULL) {
    [FrontBaseException raise:@"FBException"
                        format:@"couldn't connect db %@@%@", _db, _hostName];
  }
}

- (void)close {
  if (self->fb) {
    fbcdcClose(self->fb);
    self->fb = NULL;
  }
}

- (void *)fbcConnection {
  return self->fb;
}

- (FBMetaData *)disconnectWithSessionID:(NSString *)_sid {
  FBCMetaData *md;

  if ((md = fbcdcDisconnect(self->fb, (char *)[_sid cString])) == NULL) {
    [FrontBaseException raise:@"FBException"
                        format:@"Couldn't create metadata object !"];
    return nil;
  }
  return AUTORELEASE([[FBMetaData alloc] initWithSession:md connection:self]);
}
- (FBMetaData *)disconnectAll {
  FBCMetaData *md;

  if ((md = fbcdcDisconnectAll(self->fb)) == NULL) {
    [FrontBaseException raise:@"FBException"
                        format:@"Couldn't create metadata object !"];
    return nil;
  }
  return AUTORELEASE([[FBMetaData alloc] initWithSession:md connection:self]);
}
- (FBMetaData *)disconnectDefault {
  FBCMetaData *md;

  if ((md = fbcdcDisconnectDefault(self->fb)) == NULL) {
    [FrontBaseException raise:@"FBException"
                        format:@"Couldn't create metadata object !"];
    return nil;
  }
  return AUTORELEASE([[FBMetaData alloc] initWithSession:md connection:self]);
}
- (FBMetaData *)disconnectCurrent {
  FBCMetaData *md;

  if ((md = fbcdcDisconnectCurrent(self->fb)) == NULL) {
    [FrontBaseException raise:@"FBException"
                        format:@"Couldn't create metadata object !"];
    return nil;
  }
  return AUTORELEASE([[FBMetaData alloc] initWithSession:md connection:self]);
}

- (FBMetaData *)setConnection:(NSString *)_sid {
  FBCMetaData *md;

  if ((md = fbcdcSetConnection(self->fb, (char *)[_sid cString])) == NULL) {
    [FrontBaseException raise:@"FBException"
                        format:@"Couldn't create metadata object !"];
    return nil;
  }
  return AUTORELEASE([[FBMetaData alloc] initWithSession:md connection:self]);
}
- (FBMetaData *)setConnectionToDefault {
  FBCMetaData *md;

  if ((md = fbcdcSetConnectionToDefault(self->fb)) == NULL) {
    [FrontBaseException raise:@"FBException"
                        format:@"Couldn't create metadata object !"];
    return nil;
  }
  return AUTORELEASE([[FBMetaData alloc] initWithSession:md connection:self]);
}

/* sessions */

- (FBMetaData *)createSession:(NSString *)_sn
  sessionUser:(NSString *)_user
  password:(NSString *)_pwd
  systemUser:(NSString *)_login
{
  FBCMetaData *md;

  md = fbcdcCreateSession(self->fb,
                          (char *)[_sn    cString],
                          (char *)[_user  cString],
                          (char *)[_pwd   cString],
                          (char *)[_login cString]);

  if (md == NULL) {
    [FrontBaseException raise:@"FBException"
                        format:@"couldn't create session object !"];
    return nil;
  }

  if (fbcmdErrorCount(md) > 0) {
    NSString *error;
    char *cerror;

    cerror = fbcemdAllErrorMessages(fbcdcErrorMetaData(self->fb, md));
    error = [NSString stringWithCString:cerror];
    free(cerror); cerror = NULL;
    fbcmdRelease(md); md = NULL;

    [FrontBaseException raise:@"FBException"
                        format:@"Session creation error: %@", error];
    return nil;
  }

  return AUTORELEASE([[FBMetaData alloc] initWithSession:md connection:self]);
}

- (FBMetaData *)createSessionWithUser:(NSString *)_user 
  password:(NSString *)_pwd
{
  return [self createSession:[NSString stringWithFormat:@"%i@%@",
                                         getpid(), [[NSHost currentHost] name]]
               sessionUser:_user
               password:_pwd
               systemUser:NSUserName()];
}

/* SQL execution */

- (FBMetaData *)executeDirectSQL:(NSString *)_sql {
  FBCMetaData *md;

  if (_sql == nil) return nil;

  NSLog(@"executing %@", _sql);

  if ((md = fbcdcExecuteDirectSQL(self->fb, (char *)[_sql cString]))) {
    if (fbcmdErrorCount(md) > 0) {
      FBCErrorMetaData *emd;
      NSString *error;
      char *cerror;

      emd    = fbcdcErrorMetaData(self->fb, md);
      cerror = fbcemdAllErrorMessages(emd);
      error  = [NSString stringWithCString:cerror];
      free(cerror);       cerror = NULL;
      fbcmdRelease(md);   md     = NULL;
      fbcemdRelease(emd); emd    = NULL;

      [FrontBaseException raise:@"FBException"
                          format:@"SQL Error: %@", error];
      return nil;
    }

    return AUTORELEASE([[FBMetaData alloc] initWithSession:md connection:self]);
  }
  else {
    [FrontBaseException raise:@"FBException"
                        format:@"couldn't create metadata object !"];
    return nil;
  }
}

/* fetching */

- (NSValue *)fetch:(unsigned)_rowCount handle:(NSString *)_handle {
  void *raw;

  if ((raw = fbcdcFetch(self->fb, _rowCount, (char *)[_handle cString])))
    /* return data */
    return [NSValue valueWithPointer:raw];
  else
    /* no more rows */
    return nil;
}

- (NSValue *)fetchWithHandle:(NSString *)_handle {
  return [self fetch:1 handle:_handle];
}

- (FBMetaData *)cancelFetch:(NSString *)_handle {
  FBCMetaData *md;

  if ((md = fbcdcCancelFetch(self->fb, (char *)[_handle cString])) == NULL) {
    [FrontBaseException raise:@"FBException"
                        format:@"couldn't create metadata object !"];
    return nil;
  }

  return AUTORELEASE([[FBMetaData alloc] initWithSession:md connection:self]);
}

/* transactions */

- (FBMetaData *)commit {
  FBCMetaData *md;

  if ((md = fbcdcCommit(self->fb)) == NULL) {
    [FrontBaseException raise:@"FBException"
                        format:@"commit: Couldn't create metadata object !"];
    return nil;
  }

  return AUTORELEASE([[FBMetaData alloc] initWithSession:md connection:self]);
}
- (FBMetaData *)rollback {
  FBCMetaData *md;

  if ((md = fbcdcRollback(self->fb)) == NULL) {
    [FrontBaseException raise:@"FBException"
                        format:@"rollback: Couldn't create metadata object !"];
    return nil;
  }

  return AUTORELEASE([[FBMetaData alloc] initWithSession:md connection:self]);
}

/* replication stuff */

- (unsigned)transactionVersion {
  return fbcdcTransactionVersion(self->fb);
}

- (FBMetaData *)setTransactionVersion:(unsigned)_version {
  FBCMetaData *md;

  if ((md = fbcdcSetTransactionVersion(self->fb, _version)) == NULL) {
    [FrontBaseException raise:@"FBException"
                        format:@"setTxVer: Couldn't create metadata object !"];
    return nil;
  }
  return AUTORELEASE([[FBMetaData alloc] initWithSession:md connection:self]);
}

- (FBMetaData *)setSessionDefaultsFromReplicator:(NSString *)_defaults {
  FBCMetaData *md;

  if ((md = fbcdcSetSessionDefaultsFromReplicator(self->fb,
                                                  (char *)[_defaults cString]))
      == NULL) {
    [FrontBaseException raise:@"FBException"
                        format:@"SetSnDefs: Couldn't create metadata object !"];
    return nil;
  }
  return AUTORELEASE([[FBMetaData alloc] initWithSession:md connection:self]);
}

/* blobs */

- (FBBlobHandle *)writeBLOB:(NSData *)_blob {
  FBCBlobHandle *bh;

  if ((bh = fbcdcWriteBLOB(self->fb, (void*)[_blob bytes], [_blob length])))
    return AUTORELEASE([[FBBlobHandle alloc] initWithCBlob:bh]);
  else
    return nil;
}

- (NSData *)readBLOB:(FBBlobHandle *)_handle {
  return [self notImplemented:_cmd];
}

@end /* FBDatabaseConnection */
