/* 
   FBDatatypeMetaData.m

   Copyright (C) 1999 MDlink online service center GmbH and Helge Hess

   Author: Helge Hess (helge.hess@mdlink.de)

   This file is part of the FB Adaptor Library

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; see the file COPYING.LIB.
   If not, write to the Free Software Foundation,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
// $Id: FBDatatypeMetaData.m,v 1.2 1999/11/02 23:32:40 helge Exp $

#include "FBDatatypeMetaData.h"
#include <FBCAccess/FBCAccess.h>
#import <Foundation/Foundation.h>

@implementation FBDatatypeMetaData

- (id)initWithComponents:(NSString *)_str {
  if ((self->dmd = fbcdmdMetaDataWithComponents((char *)[_str cString]))==NULL) {
    NSLog(@"Couldn't create datatype meta data from '%@'", _str);
    RELEASE(self);
    return nil;
  }
  return self;
}
- (id)initWithCDatatypeMetaData:(void *)_cdmd {
  if (_cdmd == NULL) {
    NSLog(@"Missing c-datatype meta data ..");
    RELEASE(self);
    return nil;
  }
  self->dmd = _cdmd;
  return self;
}

- (void)dealloc {
  if (self->dmd) {
    fbcdmdRelease(self->dmd);
    self->dmd = NULL;
  }
  [super dealloc];
}

/* accessors */

- (void *)cDatatypeMetaData {
  return self->dmd;
}

- (unsigned)datatypeCode {
  return fbcdmdDatatypeCode(self->dmd);
}
- (unsigned)length {
  return fbcdmdLength(self->dmd);
}
- (unsigned)precision {
  return fbcdmdPrecision(self->dmd);
}

- (NSString *)datatypeString {
  char *s;

  return ((s = fbcdmdDatatypeString(self->dmd)))
    ? [NSString stringWithCString:s]
    : nil;
}

- (NSString *)from {
  char *s;

  return ((s = fbcdmdFrom(self->dmd)))
    ? [NSString stringWithCString:s]
    : nil;
}

- (NSString *)to {
  char *s;

  return ((s = fbcdmdTo(self->dmd)))
    ? [NSString stringWithCString:s]
    : nil;
}

/* description */

- (NSString *)description {
  return [NSString stringWithFormat:
                     @"<%@[0x%08X]:"
                     @" code=%d length=%d precision=%d"
                     @" string=%@ from=%@ to=%@",
                     NSStringFromClass([self class]), self,
                     [self datatypeCode],
                     [self length],
                     [self precision],
                     [self datatypeString],
                     [self from], [self to]];
}

@end /* FBDatatypeMetaData */
