/* 
   FBExecHandler.m

   Copyright (C) 1999 MDlink online service center GmbH and Helge Hess

   Author: Helge Hess (helge.hess@mdlink.de)

   This file is part of the FB Adaptor Library

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; see the file COPYING.LIB.
   If not, write to the Free Software Foundation,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
// $Id: FBExecHandler.m,v 1.3 1999/11/02 23:32:40 helge Exp $

#include "FBExecHandler.h"
#include <FBCAccess/FBCAccess.h>
#import <Foundation/Foundation.h>

@implementation FBExecHandler

- (id)initForHost:(NSString *)_hostName
  clientType:(int)_type
  clientName:(NSString *)_clientName
  clientID:(unsigned)_cid
{
  if (_hostName == nil) _hostName = @"localhost";
  
  self->exec = fbcehHandlerForHostWithClient((char *)[_hostName cString],
                                             _type,
                                             (char *)[_clientName cString],
                                             _cid);
  if (self->exec == nil) {
    NSLog(@"Couldn't create exec handler ..");
    RELEASE(self);
    return nil;
  }

  return self;
}

- (void)dealloc {
  if (self->exec) {
    fbcehRelease(self->exec);
    self->exec = NULL;
  }
  [super dealloc];
}

- (void *)cExecHandler {
  return self->exec;
}

/* errors */

+ (NSString *)classErrorMessage {
  const char *em;

  if ((em = fbcehClassErrorMessage()))
    return [NSString stringWithCString:em];
  else
    return nil;
}
- (NSString *)errorMessage {
  const char *em;

  if ((em = fbechErrorMessage(self->exec)))
    return [NSString stringWithCString:em];
  else
    return nil;
}

/* connection */

+ (void)disconnectAll {
  fbcehDisconnectAll();
}

- (void)disconnect {
  fbcehDisconnect(self->exec);
}

/* databases */

- (unsigned)portNumberForDatabaseNamed:(NSString *)_dbName {
  return fbcehPortNumberForDatabaseNamed(self->exec, (char *)[_dbName cString]);
}

- (unsigned)getClientID:(NSString *)_clientName {
  return fbcehGetClientID(self->exec, (char *)[_clientName cString]);
}

- (FBDatabaseStatus)statusForDatabaseNamed:(NSString *)_dbName {
  return fbcehStatusForDatabaseNamed(self->exec, (char *)[_dbName cString]);
}

- (FBArray *)availableDatabases {
  FBArray *fba;

  if ((fba = fbcehAvailableDatabases(self->exec))) {
    unsigned count;

    count = fbaCount(fba);

    return fba;
  }
  else
    return NULL;
}

- (BOOL)killDatabaseNamed:(NSString *)_dbName {
  return fbcehKillDatabaseNamed(self->exec, (char *)[_dbName cString]);
}
- (BOOL)deleteDatabaseNamed:(NSString *)_dbName {
  return fbcehDeleteDatabaseNamed(self->exec, (char *)[_dbName cString]);
}

- (BOOL)startDatabaseNamed:(NSString *)_dbName {
  return fbcehStartDatabaseNamed(self->exec, (char *)[_dbName cString]);
}
- (BOOL)startDatabaseNamed:(NSString *)_dbName options:(NSString *)_opts {
  return fbcehStartDatabaseNamedWithOptions(self->exec,
                                            (char *)[_dbName cString],
                                            (char *)[_opts cString]);
}

- (BOOL)createDatabaseNamed:(NSString *)_dbName {
  return fbcehCreateDatabaseNamed(self->exec, (char *)[_dbName cString]);
}
- (BOOL)createDatabaseNamed:(NSString *)_dbName options:(NSString *)_opts {
  return fbcehCreateDatabaseNamedWithOptions(self->exec,
                                             (char *)[_dbName cString],
                                             (char *)[_opts cString]);
}

/* licenses */

- (BOOL)setLicenseString:(NSString *)_str {
  return fbcehSetLicenseString(self->exec, (char *)[_str cString]);
}

- (NSString *)licenseString:(NSString *)_str {
  const char *ls;

  if ((ls = fbcehLicenseString(self->exec, (char *)[_str cString])))
    return [NSString stringWithCString:ls];
  else
    return nil;
}

@end /* FBExecHandler */
