/* 
   FBMetaData.m

   Copyright (C) 1999 MDlink online service center GmbH and Helge Hess

   Author: Helge Hess (helge.hess@mdlink.de)

   This file is part of the FB Adaptor Library

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; see the file COPYING.LIB.
   If not, write to the Free Software Foundation,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
// $Id: FBMetaData.m,v 1.2 1999/11/02 23:32:41 helge Exp $

#include "FBMetaData.h"
#include "FBDatabaseConnection.h"
#include "FBColumnMetaData.h"
#include "FBDatatypeMetaData.h"
#include "FBBlobHandle.h"
#include <FBCAccess/FBCAccess.h>
#import <Foundation/Foundation.h>

@implementation FBMetaData

- (id)initWithSession:(void *)_sn connection:(FBDatabaseConnection *)_conn {
  self->md         = _sn;
  self->connection = RETAIN(_conn);
  return self;
}

- (void)dealloc {
  if (self->md) {
    fbcmdRelease(self->md);
    self->md = NULL;
  }
    
  RELEASE(self->connection);
  [super dealloc];
}

- (void *)cMetaData {
  return self->md;
}

/* fetch info */

- (unsigned)columnCount {
  return fbcmdColumnCount(self->md);
}
- (int)rowCount {
  return fbcmdRowCount(self->md);
}

- (unsigned)batchSize {
  return fbcmdBatchSize(self->md);
}
- (unsigned)uniqueKey {
  return fbcmdUniqueKey(self->md);
}
- (unsigned)transactionVersion {
  return fbcmdTransactionVersion(self->md);
}

/* further meta data */

- (FBColumnMetaData *)columnMetaDataAtIndex:(unsigned)_idx {
  FBCColumnMetaData *cmd;

  if ((cmd = fbcmdColumnMetaDataAtIndex(self->md, _idx))) {
    FBColumnMetaData *ocmd;

    ocmd = [[FBColumnMetaData alloc] initWithCColumnMetaData:cmd];
    AUTORELEASE(ocmd);

    return ocmd;
  }
  else
    return nil;
}
- (NSArray *)columnMetaData {
  int count = [self columnCount];
  
  if (count > 0) {
    id objects[count];
    id array;
    int i;

    for (i = 0; i < count; i++) {
      FBCColumnMetaData *cmd;

      cmd = fbcmdColumnMetaDataAtIndex(self->md, i);
      
      objects[i] = [[FBColumnMetaData alloc] initWithCColumnMetaData:cmd];
    }

    array = [NSArray arrayWithObjects:objects count:count];
    for (i = 0; i < count; i++)
      RELEASE(objects[i]);

    return array;
  }
  else
    return [NSArray array];
}

- (FBDatatypeMetaData *)datatypeMetaDataAtIndex:(unsigned)_idx {
  FBCDatatypeMetaData *cmd;

  if ((cmd = fbcmdDatatypeMetaDataAtIndex(self->md, _idx))) {
    FBDatatypeMetaData *ocmd;

    ocmd = [[FBDatatypeMetaData alloc] initWithCDatatypeMetaData:cmd];
    AUTORELEASE(ocmd);

    return ocmd;
  }
  else
    return nil;
}
- (NSArray *)datatypeMetaData {
  int count = [self columnCount];
  
  if (count > 0) {
    id objects[count];
    id array;
    int i;

    for (i = 0; i < count; i++) {
      FBCDatatypeMetaData *cmd;

      cmd = fbcmdDatatypeMetaDataAtIndex(self->md, i);
      
      objects[i] = [[FBDatatypeMetaData alloc] initWithCDatatypeMetaData:cmd];
    }

    array = [NSArray arrayWithObjects:objects count:count];
    for (i = 0; i < count; i++)
      RELEASE(objects[i]);

    return array;
  }
  else
    return [NSArray array];
}

/* error handling */

- (BOOL)errorsFound {
  return fbcmdErrorsFound(self->md);
}
- (unsigned)errorCount {
  return fbcmdErrorCount(self->md);
}

- (FBCErrorMetaData *)_fbcErrorMetaData {
  if (self->md == NULL) return NULL;

  if (![self errorsFound])
    return NULL;
  
  return fbcdcErrorMetaData([self->connection fbcConnection], self->md);
}

- (NSString *)allErrorMessages {
  FBCErrorMetaData *emd;
  NSString *error;
  char *cerror;

  if (![self errorsFound])
    return nil;

  emd    = [self _fbcErrorMetaData];
  cerror = fbcemdAllErrorMessages(emd);
  error  = [NSString stringWithCString:cerror];
  free(cerror); cerror = NULL;
  fbcemdRelease(emd); emd = NULL;

  return error;
}

/* general info */

- (NSString *)catalogName {
  return [NSString stringWithCString:fbcmdCatalogName(self->md)];
}

- (NSString *)message {
  return [NSString stringWithCString:fbcmdMessage(self->md)];
}

- (NSTimeInterval)executionTime {
  return atof(fbcmdExecutionTime(self->md));
}

- (NSString *)fetchHandle {
  return [NSString stringWithCString:fbcmdFetchHandle(self->md)];
}

- (NSString *)statementHandle {
  return [NSString stringWithCString:fbcmdStatementHandle(self->md)];
}

- (NSString *)statementType {
  return [NSString stringWithCString:fbcmdStatementType(self->md)];
}

- (NSString *)errorClass {
  return [NSString stringWithCString:fbcmdErrorClass(self->md)];
}

/* blob handle */

- (FBBlobHandle *)blobHandle {
  FBCBlobHandle *fbcBlob;

  if ((fbcBlob = fbcmdBlobHandle(self->md))) {
    FBBlobHandle *blob;

    blob = [[FBBlobHandle alloc] initWithCBlob:fbcBlob];
    return AUTORELEASE(blob);
  }
  else
    return nil;
}

@end /* FBMetaData */
