/* 
   NSArray+FBArray.m

   Copyright (C) 1999 MDlink online service center GmbH and Helge Hess

   Author: Helge Hess (helge.hess@mdlink.de)

   This file is part of the FB Adaptor Library

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; see the file COPYING.LIB.
   If not, write to the Free Software Foundation,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
// $Id: NSArray+FBArray.m,v 1.2 1999/11/02 23:32:42 helge Exp $

#include <FBCAccess/FBCAccess.h>
#include <FBCAccess/FBChar.h>
#import <Foundation/Foundation.h>

@implementation NSArray(FBArrayAdditions)

+ (id)_arrayWithFBArrayChar:(FBArray *)_fba count:(unsigned)_count {
  id objects[_count];
  int i;

  for (i = 0; i < _count; i++) {
    char *value;

    value = (char *)fbaObjectAtIndex(_fba, i);
    NSAssert(value, @"NULL value ..");
    
    objects[i] = [NSNumber numberWithChar:*value];
  }
  
  return [NSArray arrayWithObjects:objects count:_count];
}

+ (id)_arrayWithFBArrayInt:(FBArray *)_fba count:(unsigned)_count {
  id objects[_count];
  int i;

  for (i = 0; i < _count; i++) {
    int *value;

    value = (int *)fbaObjectAtIndex(_fba, i);
    NSAssert(value, @"NULL value ..");
    
    objects[i] = [NSNumber numberWithInt:*value];
  }
  
  return [NSArray arrayWithObjects:objects count:_count];
}

+ (id)_arrayWithFBArrayUnsigned:(FBArray *)_fba count:(unsigned)_count {
  id objects[_count];
  int i;

  for (i = 0; i < _count; i++) {
    unsigned *value;

    value = (unsigned *)fbaObjectAtIndex(_fba, i);
    NSAssert(value, @"NULL value ..");
    
    objects[i] = [NSNumber numberWithUnsignedInt:*value];
  }
  
  return [NSArray arrayWithObjects:objects count:_count];
}

+ (id)_arrayWithFBArrayFBChar:(FBArray *)_fba count:(unsigned)_count {
  id objects[_count];
  int i;

  for (i = 0; i < _count; i++) {
    FBChar **value;

    value = (FBChar **)fbaObjectAtIndex(_fba, i);
    NSAssert(value, @"NULL value ..");
    
    objects[i] = [NSString stringWithCString:*value
                           length:fbcharSize(*value)];
  }
  
  return [NSArray arrayWithObjects:objects count:_count];
}

+ (id)arrayWithFBArray:(FBArray *)_fba {
  unsigned count;

  if (_fba == NULL)
    return nil;

  if ((count = fbaCount(_fba)) == 0) {
    return [NSArray array];
  }
  else {
    switch (fbaObjectType(_fba)) {
      case FBAOT_char:
        return [self _arrayWithFBArrayChar:_fba count:count];

      case FBAOT_int:
        return [self _arrayWithFBArrayInt:_fba count:count];

      case FBAOT_unsigned:
        return [self _arrayWithFBArrayUnsigned:_fba count:count];

      case FBAOT_FBChar:
        return [self _arrayWithFBArrayFBChar:_fba count:count];

      case FBAOT_FBPrimaryKey:
      case FBAOT_FBArray:
      case FBAOT_Row:
      case FBAOT_Unknown:
      case FBAOT_Object:
      case FBAOT_IL1:
      case FBAOT_IL2:
        NSLog(@"cannot handle array value of type %i", fbaObjectType(_fba));
        return nil;
    }
  }
}

@end
