/* 
   FBChannel+Model.m

   Copyright (C) 1999 MDlink online service center GmbH and Helge Hess

   Author: Helge Hess (helge@mdlink.de)

   This file is part of the FB Adaptor Library

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public
   License along with this library; see the file COPYING.LIB.
   If not, write to the Free Software Foundation,
   59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
// $Id: FBChannel+Model.m,v 1.5 1999/11/02 23:32:22 helge Exp $

#import "common.h"

@implementation FrontBaseChannel(ModelFetching)

- (NSArray *)_attributesForTableName:(NSString *)_tableName {
  NSMutableArray      *attributes;
  NSArray             *resultDescription;
  NSString            *selectExpression;
  NSString            *columnNameKey;
  NSString            *externalTypeKey;
  NSMutableDictionary *userType2storageType;
  NSDictionary        *row;
  int                 cnt;

  selectExpression = [NSString stringWithFormat:
                                 @"SELECT c.name, t.name "
                                 @"FROM syscolumns c, systypes t "
                                 @"WHERE c.id = object_id('%s') "
                                 @"  AND c.usertype *= t.usertype",
                                 [_tableName cString]];

  if (![self evaluateExpression:selectExpression])
    return nil;

  resultDescription  = [self describeResults];
  columnNameKey      = [(EOAttribute *)[resultDescription objectAtIndex:0] name];
  externalTypeKey    = [(EOAttribute *)[resultDescription objectAtIndex:1] name];

  attributes = [NSMutableArray arrayWithCapacity:16];

  while ((row = [self fetchAttributes:resultDescription withZone:nil])) {
    EOAttribute *attribute    = [EOAttribute new];
    NSString    *columnName   = [row objectForKey:columnNameKey];
    NSString    *externalType = [row objectForKey:externalTypeKey];
    NSString    *attrName     = [columnName _sybModelMakeInstanceVarName];
    int fbType       =
      [(FrontBaseAdaptor *)[adaptorContext adaptor]
                           typeCodeForExternalName:externalType];

    [attribute setName:attrName];
    [attribute setColumnName:columnName];
    [attribute loadValueClassAndTypeFromFrontBaseType:fbType];
    [attribute setExternalType:externalType];

    [attributes addObject:attribute];
  }

  // fetch external types

  userType2storageType = [NSMutableDictionary dictionaryWithCapacity:4];
  
  for (cnt = 0; cnt < [attributes count]; cnt++) {
    EOAttribute *attribute    = [attributes objectAtIndex:cnt];
    NSString    *externalType = [attribute externalType];
    int fbType       =
      [(FrontBaseAdaptor *)[adaptorContext adaptor]
                           typeCodeForExternalName:externalType];

    if ((fbType >= 100) || (fbType < 1)) {
      NSString *cacheType = [userType2storageType objectForKey:externalType];

      if (cacheType == nil) {
        selectExpression = [NSString stringWithFormat:@"sp_help %s",
                                     [externalType cString]];
        if ([self evaluateExpression:selectExpression]) {
          NSString *attrName = nil;
          
          resultDescription = [self describeResults];
          row = [self fetchAttributes:resultDescription withZone:nil];
          [self cancelFetch];

          attrName = [(EOAttribute *)[resultDescription objectAtIndex:1] name];

          cacheType = [row objectForKey:attrName];

          cacheType = [cacheType _sybStripEndSpaces];
          [userType2storageType setObject:cacheType forKey:externalType];

          if (isDebuggingEnabled)
            NSLog(@"type is %@ storage %@ fb %i",
                  externalType, cacheType, fbType);
        }
      }
    
      fbType =
        [(FrontBaseAdaptor *)[adaptorContext adaptor]
                             typeCodeForExternalName:cacheType];
      [attribute loadValueClassAndTypeFromFrontBaseType:fbType];
      [attribute setExternalType:externalType];
    }
  }

  return attributes;
}

- (NSArray *)_primaryKeysNamesForTableName:(NSString *)_tableName {
  NSMutableArray *primaryKeys;
  NSString       *selectExpression;
  NSArray        *resultDescription;
  NSString       *columnNameKey;
  NSString       *keyQual;
  NSDictionary   *row;

  keyQual = @"((k.key1=c.colid) OR (k.key2=c.colid) OR "
            @" (k.key3=c.colid) OR (k.key4=c.colid))";

  selectExpression = [NSString stringWithFormat:
                                 @"SELECT c.name "
                                 @"FROM syskeys k,syscolumns c "
                                 @"WHERE k.id=object_id('%s') AND "
                                 @"(k.id=c.id) AND %s AND "
                                 @"(k.type=1)", // primary keys
                                 [_tableName cString],
                                 [keyQual cString]];

  if (![self evaluateExpression:selectExpression])
    return nil;
  
  resultDescription  = [self describeResults];
  columnNameKey      = [(EOAttribute *)[resultDescription objectAtIndex:0] name];

  primaryKeys = [NSMutableArray arrayWithCapacity:4];
  
  while ((row = [self fetchAttributes:resultDescription withZone:nil]))
    [primaryKeys addObject:[row objectForKey:columnNameKey]];

  return primaryKeys;
}

- (NSArray *)_foreignKeysForTableName:(NSString *)_tableName {
  // does not work for compound keys !
  NSMutableArray *foreignKeys;
  NSString       *selectExpression;
  NSArray        *resultDescription;
  NSString       *sourceColumnKey;
  NSString       *targetColumnKey;
  NSString       *targetTableKey;
  NSDictionary   *row;

  selectExpression = [NSString stringWithFormat:
                               @"SELECT sc.name,tc.name,object_name(r.reftabid) "
                               @"FROM sysreferences r,syscolumns sc,syscolumns tc"
                               @" WHERE r.tableid=object_id('%s') AND "
                               @"(r.tableid=sc.id) AND (r.reftabid=tc.id) AND "
                               @"(r.fokey1=sc.colid) AND (r.refkey1=tc.colid)",
                               [_tableName cString]];
  /*
  selectExpression = [NSString stringWithFormat:
                                 @"SELECT sc.name,tc.name,object_name(k.depid) "
                                 @"FROM syskeys k,syscolumns sc,syscolumns tc "
                                 @"WHERE k.id=object_id('%s') AND "
                                 @"(k.id=sc.id) AND (k.depid=tc.id) AND"
                                 @"(k.key1=sc.colid) AND "
                                 @"(k.depkey1=tc.colid) AND "
                                 @"(k.type=2)", // foreign keys
                                 [_tableName cString]];
                                 */
  
  if (![self evaluateExpression:selectExpression])
    return nil;
  
  resultDescription = [self describeResults];
  sourceColumnKey   = [(EOAttribute *)[resultDescription objectAtIndex:0] name];
  targetColumnKey   = [(EOAttribute *)[resultDescription objectAtIndex:1] name];
  targetTableKey    = [(EOAttribute *)[resultDescription objectAtIndex:2] name];

  foreignKeys = [NSMutableArray arrayWithCapacity:4];
  
  while ((row = [self fetchAttributes:resultDescription withZone:nil])) {
    NSMutableDictionary *keyDesc =
      [NSMutableDictionary dictionaryWithCapacity:4];

    [keyDesc setObject:[row objectForKey:sourceColumnKey] forKey:@"sourceAttr"];
    [keyDesc setObject:[row objectForKey:targetColumnKey] forKey:@"targetAttr"];
    [keyDesc setObject:[row objectForKey:targetTableKey] forKey:@"targetTable"];
    [foreignKeys addObject:keyDesc];
  }

  return foreignKeys;
}

- (EOModel *)describeModelWithTableNames:(NSArray *)_tableNames {
  NSMutableArray *buildRelShips = [NSMutableArray arrayWithCapacity:64];
  EOModel *model = AUTORELEASE([EOModel new]);
  int cnt, tc = [_tableNames count];

  for (cnt = 0; cnt < tc; cnt++) {
    NSMutableDictionary *relNamesUsed         = [NSMutableDictionary new];
    NSMutableArray      *classProperties      = [NSMutableArray new];
    NSMutableArray      *primaryKeyAttributes = [NSMutableArray new];
    NSString *tableName  = [_tableNames objectAtIndex:cnt];
    NSArray  *attributes = [self _attributesForTableName:tableName];
    NSArray  *pkeys      = [self _primaryKeysNamesForTableName:tableName];
    NSArray  *fkeys      = [self _foreignKeysForTableName:tableName];
    EOEntity *entity     = AUTORELEASE([EOEntity new]);
    int      cnt2;
    int      ac  = [attributes count];
    int      fkc = [fkeys      count];

    [entity setName:[tableName _sybModelMakeClassName]];
    [entity setClassName:
              [@"EO" stringByAppendingString:[tableName _sybModelMakeClassName]]];
    [entity setExternalName:tableName];
    [classProperties addObjectsFromArray:[entity classProperties]];
    [primaryKeyAttributes addObjectsFromArray:[entity primaryKeyAttributes]];
    [model addEntity:entity];
    
    for (cnt2 = 0; cnt2 < ac; cnt2++) {
      EOAttribute *attribute  = [attributes objectAtIndex:cnt2];
      NSString    *columnName = [attribute columnName];

      [entity addAttribute:attribute];
      [classProperties addObject:attribute];

      if ([pkeys containsObject:columnName])
        [primaryKeyAttributes addObject:attribute];
    }
    [entity setClassProperties:classProperties];
    [entity setPrimaryKeyAttributes:primaryKeyAttributes];
    
    for (cnt2 = 0; cnt2 < fkc; cnt2++) {
      NSDictionary   *fkey             = [fkeys objectAtIndex:cnt2];
      NSMutableArray *classProperties  = [NSMutableArray new];
      NSString       *sa               = [fkey objectForKey:@"sourceAttr"];
      NSString       *da               = [fkey objectForKey:@"targetAttr"];
      NSString       *dt               = [fkey objectForKey:@"targetTable"];
      EORelationship *rel              = [EORelationship new];
      EOJoin         *join             = [EOJoin new];
      NSString       *relName          = nil;

      if ([pkeys containsObject:sa])
        relName = [@"to" stringByAppendingString:[dt _sybModelMakeClassName]];
      else {
        relName = [@"to" stringByAppendingString:
                    [[sa _sybModelMakeInstanceVarName]
                         _sybStringWithCapitalizedFirstChar]];
        if ([relName hasSuffix:@"Id"]) {
          int cLength = [relName cStringLength];

          relName = [relName substringToIndex:cLength - 2];
        }
      }
      if ([relNamesUsed objectForKey:relName]) {
        int useCount = [[relNamesUsed objectForKey:relName] intValue];
        
        [relNamesUsed setObject:[NSNumber numberWithInt:(useCount++)] forKey:relName];
        relName = [NSString stringWithFormat:@"%s%d",
                              [relName cString], useCount];
      }
      else
        [relNamesUsed setObject:[NSNumber numberWithInt:0] forKey:relName];

      [rel setName:relName];
      //[rel setDestinationEntity:(EOEntity *)[dt _sybModelMakeClassName]];
      [rel setToMany:NO];

      [join setSourceAttribute:
              (EOAttribute *)[sa _sybModelMakeInstanceVarName]];
      [join setDestinationAttribute:
              (EOAttribute *)[da _sybModelMakeInstanceVarName]];
      [rel  addJoin:join];
      
      [entity addRelationship:rel];
      [classProperties addObjectsFromArray:[entity classProperties]];
      [classProperties addObject:rel];
      [entity setClassProperties:classProperties];
      [buildRelShips addObject:rel];
    }

    [entity setAttributesUsedForLocking:[[entity attributes] copy]];
  }

  [buildRelShips makeObjectsPerform:@selector(replaceStringsWithObjects)];
  /*
  // make reverse relations
  {
    int cnt, rc = [buildRelShips count];

    for (cnt = 0; cnt < rc; cnt++) {
      EORelationship *rel     = [buildRelShips objectAtIndex:cnt];
      NSMutableArray *classProperties  = [NSMutableArray new];   
      EORelationship *reverse = [rel reversedRelationShip];
      EOEntity       *entity  = [rel destinationEntity];
      NSArray        *pkeys   = [entity primaryKeyAttributes];
      BOOL           isToMany = [reverse isToMany];
      EOAttribute    *sa      = [[[reverse joins] lastObject] sourceAttribute];
      NSString       *relName = nil;

      if ([pkeys containsObject:sa]
          || isToMany)
        relName = [@"to" stringByAppendingString:
                    [(EOEntity *)[reverse destinationEntity] name]];
      else {
        relName = [@"to" stringByAppendingString:
                    [[[sa name] _sybModelMakeInstanceVarName]
                          _sybStringWithCapitalizedFirstChar]];
        if ([relName hasSuffix:@"Id"]) {
          int cLength = [relName cStringLength];

          relName = [relName substringToIndex:cLength - 2];
        }
      }

      if ([entity relationshipNamed:relName]) {
        int cnt = 1;
        NSString *numName;

        numName = [NSString stringWithFormat:@"%s%d", [relName cString], cnt];
        while ([entity relationshipNamed:numName]) {
          cnt++;
          numName = [NSString stringWithFormat:@"%s%d", [relName cString], cnt];
        }

        relName = numName;
      }

      [reverse setName:relName];

      [entity addRelationship:reverse];
      
      [classProperties addObjectsFromArray:[entity classProperties]];
      [classProperties addObject:reverse];
      [entity setClassProperties:classProperties];
    }
  }
  */
  [model setAdaptorName:@"Sybase"];
  [model setAdaptorClassName:@"FBAdaptor"];
  [model setConnectionDictionary:[[adaptorContext adaptor] connectionDictionary]];
  
  return model;
}

- (NSArray *)describeTableNames {
  NSMutableArray *tableNames = nil;
  NSArray        *resultDescription;
  NSString       *attributeName;
  NSDictionary   *row;

  if (![self evaluateExpression:
               @"SELECT name FROM sysobjects WHERE type='U' OR type='V'"])
    return nil;

  resultDescription = [self describeResults];
  attributeName = [(EOAttribute *)[resultDescription objectAtIndex:0] name];

  tableNames = [NSMutableArray arrayWithCapacity:16];

  while ((row = [self fetchAttributes:resultDescription withZone:nil]))
    [tableNames addObject:[row objectForKey:attributeName]];
  
  return tableNames;
}

@end

void __link_FBChannelModel() {
  // used to force linking of object file
  __link_FBChannelModel();
}
