// $Id: fbtest.m,v 1.7 1999/11/02 13:01:30 helge Exp $

#import <Foundation/Foundation.h>
#import <NGExtensions/NGExtensions.h>
#import <NGStreams/NGStreams.h>
#import <GDLExtensions/GDLExtensions.h>
#import <eoaccess/eoaccess.h>

int main(int argc, char **argv, char **env) {
  EOModel *m;
  EOAdaptor *a;
  EOAdaptorContext *ctx;
  EOAdaptorChannel *ch;
  NSDictionary *conDict;
  NSString *expr;
  
  [NSProcessInfo initializeWithArguments:argv count:argc environment:env];

  conDict = [NSDictionary dictionaryWithContentsOfFile:@"condict.plist"];

  a = [EOAdaptor adaptorWithName:@"FrontBase"];
  [a setConnectionDictionary:conDict];
  NSLog(@"got adaptor %@", a);
  
  ctx = [a createAdaptorContext];
  ch  = [ctx createAdaptorChannel];

  m = AUTORELEASE([[EOModel alloc] initWithContentsOfFile:@"test.eomodel"]);
  [a setModel:m];

  expr = [[NSUserDefaults standardUserDefaults] stringForKey:@"sql"];

  NSLog(@"opening channel ..");

  [ch setDebugEnabled:YES];
  
  if ([ch openChannel]) {
    NSLog(@"channel is open");
    
    if ([ctx beginTransaction]) {
      NSLog(@"began tx ..");

      /* do something */
      {
        NSAutoreleasePool *pool = [NSAutoreleasePool new];
        EOEntity *e;
        EOQualifier *q;
        NSArray *attrs;

        /* fetch some person records */

        e = [m entityNamed:@"Person"];
        q = [e qualifier];
        attrs = [e attributes];

        if ([ch selectAttributes:attrs
                describedByQualifier:q
                fetchOrder:nil
                lock:NO]) {
          NSDictionary *record;

          while ((record = [ch fetchAttributes:attrs withZone:nil])) {
            NSLog(@"fetched %@ birthday %@",
                  [record valueForKey:@"description"],
                  [record valueForKey:@"companyId"]);
          }
        }

        /* fetch some team records */

        e = [m entityNamed:@"Team"];
        q = [e qualifier];
        attrs = [e attributes];

        if ([ch selectAttributes:attrs
                describedByQualifier:q
                fetchOrder:nil
                lock:NO]) {
          NSDictionary *record;

          while ((record = [ch fetchAttributes:attrs withZone:nil])) {
            NSLog(@"fetched %@ birthday %@",
                  [record valueForKey:@"description"],
                  [record valueForKey:@"companyId"]);
          }
        }

        /* do some update */

        e = [m entityNamed:@"Person"];
        attrs = [e attributes];
        q = [[EOQualifier alloc] initWithFormat:@"%A='helge'", @"login"];
        AUTORELEASE(q);

        if ([ch selectAttributes:attrs
                describedByQualifier:q
                fetchOrder:nil
                lock:NO]) {
          NSDictionary *record;

          record = [ch fetchAttributes:attrs withZone:nil];
        }
#if 0
        /* fetch some expr */

        if (expr) {
          if ([ch evaluateExpression:expr]) {
            NSDictionary *record;

            attrs = [ch describeResults];
            //NSLog(@"results: %@", attrs);

            while ((record = [ch fetchAttributes:attrs withZone:nil]))
              NSLog(@"fetched %@", record);
          }
        }
#endif
        
        RELEASE(pool);
      }

      NSLog(@"committing tx ..");
      if ([ctx commitTransaction])
        NSLog(@"  could commit.");
      else
        NSLog(@"  commit failed.");
    }
    
    NSLog(@"closing channel ..");
    [ch closeChannel];
  }

  return 0;
}
